"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tipo_mensagem_1 = require("./modelo/tipo-mensagem");
var PostMessageApi = /** @class */ (function () {
    function PostMessageApi() {
        var _this = this;
        this.receberMensagem = function (evento) {
            /*
             *  Fix duplicação de recebimento de evento do PostMessage
             *  quando é utilizado com o Webpack Server.
             *   https://github.com/webpack/webpack-dev-server/issues/481
             */
            if (!evento.data || evento.data.type)
                return;
            _this.tratarMensagemRecebida(evento.data);
        };
        this.adicionarEventListenerReceberMensagem();
    }

    PostMessageApi.prototype.adicionarEventListenerReceberMensagem = function () {
        window.addEventListener("message", this.receberMensagem, false);
    };

    PostMessageApi.prototype.enviarMensagem = function (mensagem) {
        window.parent.postMessage(mensagem, '*');
    };

    PostMessageApi.prototype.tratarMensagemRecebida = function (mensagem) {
        var event = null;
        switch (mensagem.tipo) {
            case tipo_mensagem_1.TipoMensagemAPI.OBTEM_DADOS_GENERICOS:
                event = new CustomEvent('evObtemDadosGenericos', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.OBTEM_ULTIMA_PAGINA_ACESSADA:
                event = new CustomEvent('evObtemUltimaPaginaAcessada', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.OBTEM_DADOS_CONCLUSAO_UNIDADE:
                event = new CustomEvent('evObtemDadosConclusaoUnidade', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.OBTEM_SE_EXISTE_PROXIMA_UNIDADE:
                event = new CustomEvent('evObtemSeExisteProximaUnidade', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.OBTEM_PROXIMA_UNIDADE:
                event = new CustomEvent('evObtemProximaUnidade', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.OBTEM_CONFIGURACAO_ATIVIDADE:
                event = new CustomEvent('evObtemConfiguracaoAtividade', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.OBTEM_RESPOSTA_ATIVIDADE:
                event = new CustomEvent('evObtemDadosRespostaAtividade', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.OBTEM_DADOS_CONCLUSAO_FORMACAO:
                event = new CustomEvent('evObtemDadosDadosConclusaoFormacao', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.OBTEM_DADOS_CONFIGURACAO_CONCLUSAO_UNIDADE:
                event = new CustomEvent('evObtemDadosConfiguracaoConclusaoUnidade', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.OBTEM_STATUS_CONTRASTE:
                event = new CustomEvent('evObtemStatusContraste', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.REGISTRA_DADOS_GENERICOS:
                event = new CustomEvent('evObtemRegistraDadosGenericos', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.REGISTRA_ULTIMA_PAGINA_ACESSADA:
                event = new CustomEvent('evObtemRegistraUltimaPaginaAcessada', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.REGISTRA_PORCENTAGEM_CONCLUSAO_UNIDADE:
                event = new CustomEvent('evRegistraPorcentagemConclusaoUnidade', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            case tipo_mensagem_1.TipoMensagemAPI.REGISTRA_RESPOSTA_ATIVIDADE:
                event = new CustomEvent('evRegistraRespostaAtividade', { detail: { status: mensagem.status, data: mensagem.objeto } });
                break;
            default:
                break;
        }
        // Dispara evento de retorno
        window.dispatchEvent(event);
    };

    return PostMessageApi;
}());
exports.PostMessageApi = PostMessageApi;
//# sourceMappingURL=post-message-api.js.map
