import {MensagemAPI} from './modelo/mensagem';
import {TipoMensagemAPI} from './modelo/tipo-mensagem';
import {HTTPStatus} from './modelo/http-status';

class Deferred {
    private promise: Promise<unknown>;
    private reject: (reason?: any) => void;
    private resolve: (value?: unknown) => void;
    constructor() {
        this.promise = new Promise((resolve, reject)=> {
            this.reject = reject;
            this.resolve = resolve;
        });
    }
}


export class PostMessageApi{

    private promises: Array<any> = [];
    private readonly debug: boolean;
    private readonly debugAVAMEC: boolean;

    constructor(config) {
        this.debug = config.debug ? config.debug : false;
        this.debugAVAMEC = config.debugAVAMEC ? config.debugAVAMEC : false;
        this.fixCustomEventIE();
        this.adicionarEventListenerReceberMensagem();
    }

    /*
    ======================================================
    uuid-v4.js :: Random UUID (v4) Generator - Usage: UUID()
    ======================================================
    Copyright (c) 2011 Matt Williams <matt@makeable.co.uk>. All rights reserved.
    Redistribution and use in source and binary forms, with or without modification, are
    permitted provided that the following conditions are met:
       1. Redistributions of source code must retain the above copyright notice, this list of
          conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice, this list
          of conditions and the following disclaimer in the documentation and/or other materials
          provided with the distribution.
    THIS SOFTWARE IS PROVIDED BY MATT WILLIAMS ''AS IS'' AND ANY EXPRESS OR IMPLIED
    WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MATT WILLIAMS OR
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
    ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    The views and conclusions contained in the software and documentation are those of the
    authors and should not be interpreted as representing official policies, either expressed
    or implied, of Matt Williams.
    */
    private  UUID() {
        let dec2hex = [];
        for (let i=0; i<=15; i++) {
            dec2hex[i] = i.toString(16);
        }
        let uuid = '';
        for (let i=1; i<=36; i++) {
            if (i===9 || i===14 || i===19 || i===24) {
                uuid += '-';
            } else if (i===15) {
                uuid += 4;
            } else if (i===20) {
                uuid += dec2hex[(Math.random()*4|0 + 8)];
            } else {
                uuid += dec2hex[(Math.random()*16|0)];
            }
        }
        return uuid;
    }

    private adicionarEventListenerReceberMensagem(): void {
        window.addEventListener("message", this.receberMensagem, false);
    }

    public enviarMensagem(mensagem: MensagemAPI): Promise<any>{
        if (this.debug) console.groupCollapsed('[AVAMEC Bridge API] Enviando mensagem: ');
        mensagem.UUID = this.UUID();
        if(this.debugAVAMEC) mensagem.log = this.debug;

        this.promises[mensagem.UUID] = new Deferred();

        if (this.debug) console.table(mensagem);
        window.parent.postMessage(mensagem, '*');

        if (this.debug) console.groupEnd();
        if(mensagem.tipo === TipoMensagemAPI.ESTOU_NO_AVAMEC) {
            this.promises[mensagem.UUID].timeout = setTimeout(() => {
                if (this.debug) console.log('[AVAMEC Bridge API] Resolveu por timeout');
                this.promises[mensagem.UUID].resolve({
                    data: false,
                    status: HTTPStatus.OK
                });
            }, 1000);
        }
        return this.promises[mensagem.UUID].promise;
    }

    private receberMensagem = (evento) => {
        /*
         *  Fix duplicação de recebimento de evento do PostMessage
         *  quando é utilizado com o Webpack Server.
         *   https://github.com/webpack/webpack-dev-server/issues/481
         */
        if (!evento.data || evento.data.type || !evento.data.status ) return;

        if (this.debug) console.groupCollapsed('[AVAMEC Bridge API] Resposta recebida: ');

        if (this.debug) console.table(evento);
        if (this.debug) console.table(evento.data);
        if(evento.data.UUID) {
            if (this.debug) console.log('[AVAMEC Bridge API] Contém UUID: ', evento.data.UUID);

            if(evento.data.tipo === TipoMensagemAPI.ESTOU_NO_AVAMEC) {
                clearTimeout(this.promises[evento.data.UUID].timeout);
            }

            if (evento.data.status === HTTPStatus.OK) {
                if (this.debug) console.log('[AVAMEC Bridge API] Status 200');
                if (this.debug) console.log('[AVAMEC Bridge API] Resolvendo promise');
                this.promises[evento.data.UUID].resolve({
                    data: evento.data.objeto,
                    status: evento.data.status
                });
            } else {
                if (this.debug) console.log(`[AVAMEC Bridge API] Status ${evento.data.status}`);
                if (this.debug) console.log('[AVAMEC Bridge API] Rejeitando promise');
                this.promises[evento.data.UUID].reject({
                    data: evento.data.objeto,
                    status: evento.data.status
                });
            }
            if (this.debug) console.log('[AVAMEC Bridge API] Disparando evento para compatibilidade');
            this.tratarMensagemRecebida(evento.data);
        } else {
            if (this.debug) console.log('[AVAMEC Bridge API] Não contém UUID');
            if (this.debug) console.log('[AVAMEC Bridge API] Disparando evento');
            this.tratarMensagemRecebida(evento.data);
        }
    };

    private tratarMensagemRecebida(mensagem: MensagemAPI){
        let event = null;
        switch (mensagem.tipo){
            case TipoMensagemAPI.OBTEM_DADOS_GENERICOS:
                event =  new CustomEvent('evObtemDadosGenericos', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_ULTIMA_PAGINA_ACESSADA:
                event =  new CustomEvent('evObtemUltimaPaginaAcessada', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_DADOS_CONCLUSAO_UNIDADE:
                event =  new CustomEvent('evObtemDadosConclusaoUnidade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_SE_EXISTE_PROXIMA_UNIDADE:
                event =  new CustomEvent('evObtemSeExisteProximaUnidade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_SE_EXISTE_UNIDADE_ANTERIOR:
                event =  new CustomEvent('evObtemSeExisteUnidadeAnterior', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_PROXIMA_UNIDADE:
                event =  new CustomEvent('evObtemProximaUnidade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_UNIDADE_ANTERIOR:
                event =  new CustomEvent('evObtemUnidadeAnterior', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_UNIDADE:
                event =  new CustomEvent('evObtemUnidade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_CONFIGURACAO_ATIVIDADE:
                event =  new CustomEvent('evObtemConfiguracaoAtividade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_RESPOSTA_ATIVIDADE:
                event =  new CustomEvent('evObtemDadosRespostaAtividade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_COMENTARIOS_ATIVIDADE:
                event =  new CustomEvent('evObtemComentariosAtividade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_DADOS_CONCLUSAO_CURSO:
                event =  new CustomEvent('evObtemDadosDadosConclusaoCurso', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_TIPO_ACESSO_UNIDADE:
                event =  new CustomEvent('evObtemTipoAcessoUnidade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_IDENTIFICADOR_USUARIO:
                event =  new CustomEvent('evObtemIdentificadorUsuarioLogado', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_DADOS_CURSO:
                event =  new CustomEvent('evObtemDadosCurso', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_DADOS_CONFIGURACAO_CONCLUSAO_UNIDADE:
                event =  new CustomEvent('evObtemDadosConfiguracaoConclusaoUnidade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_STATUS_CONTRASTE:
                event =  new CustomEvent('evObtemStatusContraste', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_TAMANHO_FONTE_SISTEMA:
                event =  new CustomEvent('evObtemTamanhoFonteSistema', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_ANCORA_IR_PARA_CONTEUDO:
                event =  new CustomEvent('evObtemAncoraIrParaConteudo', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.OBTEM_PORCENTAGEM_CONCLUSAO_UNIDADE:
                event =  new CustomEvent('evObtemPorcentagemConclusaoUnidade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.REGISTRA_DADOS_GENERICOS:
                event =  new CustomEvent('evObtemRegistraDadosGenericos', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.REGISTRA_ULTIMA_PAGINA_ACESSADA:
                event =  new CustomEvent('evObtemRegistraUltimaPaginaAcessada', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.REGISTRA_PORCENTAGEM_CONCLUSAO_UNIDADE:
                event =  new CustomEvent('evRegistraPorcentagemConclusaoUnidade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.REGISTRA_RESPOSTA_ATIVIDADE:
                event =  new CustomEvent('evRegistraRespostaAtividade', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            case TipoMensagemAPI.REGISTRA_RESPOSTA_ATIVIDADE_ARQUIVO:
                event =  new CustomEvent('evRegistraRespostaAtividadeArquivo', { detail: { status: mensagem.status, data: mensagem.objeto }});
                break;
            default:
                break;
        }

        if(!!event){
            // Dispara evento de retorno
            window.dispatchEvent(event);
        }

    }

    private fixCustomEventIE(){
        if ( typeof window.CustomEvent === "function" ) return false;

        function CustomEvent ( event, params ) {
            params = params || { bubbles: false, cancelable: false, detail: undefined };
            var evt = document.createEvent( 'CustomEvent' );
            evt.initCustomEvent( event, params.bubbles, params.cancelable, params.detail );
            return evt;
        }

        CustomEvent.prototype = window.Event.prototype;

        window.CustomEvent = CustomEvent;
    }

}

declare global {
    interface Window { CustomEvent: any; Event: any; }
}
