const path = require('path');
var WebpackAutoInject = require('webpack-auto-inject-version');

module.exports = {
    mode: "production",
    entry: {
        'bridge-rest-api': './src/index.ts'
    },
    output: {
        filename: '[name].bundle.js',
        path: path.resolve(__dirname, 'dist'),
        libraryTarget: 'umd',
        library: undefined //https://github.com/webpack/webpack/issues/3560
    },
    devServer: {
        contentBase: path.join(__dirname, "dist"),
        compress: true,
        port: 9001,
        /*host: '0.0.0.0',*/
        overlay: {
            warnings: true,
            errors: true
        }
    },
   // devtool: 'inline-source-map',
    resolve: {
        // Add `.ts` and `.tsx` as a resolvable extension.
        extensions: ['.ts', '.js']
    },
    module: {
        rules: [
            {
                test: /\.ts$/,
                enforce: 'pre',
                loader: 'tslint-loader',
                options: { tsConfigFile: 'tsconfig.json' }
            },
            // all files with a `.ts` or `.tsx` extension will be handled by `ts-loader`
            {test: /\.ts$/, loader: 'ts-loader'}
        ]
    },
    plugins: [
        new WebpackAutoInject({
            // options
            // example:
            SHORT: 'AVAMEC Bridge Rest API',
            components: {
                AutoIncreaseVersion: true,
                InjectAsComment: true
            },
            componentsOptions: {
                InjectAsComment: {
                    tag: 'Build version: {version} - {date}', // default
                    dateFormat: 'dddd, mmmm dS, yyyy, h:MM:ss TT', // default
                    multiLineCommentType: false, // default
                }
            }
        })
    ]
};
